package com.piece_framework.piece_ide.form_designer.model.defined;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.piece_framework.piece_ide.form_designer.model.Validator;

public class DefinedValidator {
    private static final List<DefinedValidator> DEFINED_VALIDATORS;

    static {
        DEFINED_VALIDATORS = new ArrayList<DefinedValidator>();

        List<DefinedRule> compareRules = new ArrayList<DefinedRule>();
        compareRules.add(new DefinedRule("to", ""));
        DEFINED_VALIDATORS.add(
                new DefinedValidator(
                        "Compare", 
                        "2 ̒lr", 
                        compareRules)); 

        List<DefinedRule> dateRules = new ArrayList<DefinedRule>();
        dateRules.add(new DefinedRule("pattern", ""));
        dateRules.add(new DefinedRule("patternYearPosition", ""));
        dateRules.add(new DefinedRule("patternMonthPosition", ""));
        dateRules.add(new DefinedRule("patternDayPostion", ""));
        DEFINED_VALIDATORS.add(
                new DefinedValidator(
                        "Date",
                        "t",
                        dateRules));

        List<DefinedRule> emailRules = new ArrayList<DefinedRule>();
        emailRules.add(new DefinedRule(
                            "allowDotBeforeAtmark", 
                            DefinedRule.WidgetType.YesNo, 
                            "false"));

        DEFINED_VALIDATORS.add(
                new DefinedValidator(
                        "Email",
                        "[AhX",
                        emailRules));
        
    }

    private String fName;
    private String fDescription;
    private List<DefinedRule> fRules;

    private DefinedValidator(
            final String name, 
            final String description,
            final List<DefinedRule> rules) {
        if (name == null) {
            throw new NullPointerException();
        }
        if (description == null) {
            throw new NullPointerException();
        }
        if (rules == null) {
            throw new NullPointerException();
        }

        fName = name;
        fDescription = description;
        fRules = new ArrayList<DefinedRule>(rules);
    }

    public String getName() {
        return fName;
    }

    public String getDescirption() {
        return fDescription;
    }

    public String getDisplayName() {
        return fName + " - " + fDescription;
    }

    public List<DefinedRule> getRules() {
        return Collections.unmodifiableList(fRules);
    }

    public static List<DefinedValidator> getList() {
        return DEFINED_VALIDATORS;
    }

    public static DefinedValidator getDefinedValidator(
                                        final Validator validator) {
        if (validator == null) {
            throw new NullPointerException();
        }

        for (DefinedValidator definedValidator : DEFINED_VALIDATORS) {
            if (definedValidator.fName.equals(validator.getName())) {
                return definedValidator;
            }
        }
        return null;
    }

    public Validator create() {
        Map<String, String> rules = new LinkedHashMap<String, String>();
        for (DefinedRule definedRule : fRules) {
            rules.put(definedRule.getName(), definedRule.getDefaultValue());
        }
        return new Validator(fName, rules);
    }
}
